/****************************************************************
* Daniel Greco
* Input.cpp will contain all the functions for receiving and handling
* user input
*
*****************************************************************/
#include "Angel.h"
#include "Input.h"
#include "Points.h"
#include "HelperFunctions.h"
#include "Lighting.h"

//externs
extern mat4 rotate, translate, projection;
extern GLuint horizAngle, vertAngle;
extern Light* lighting;
//default constructor Note:Part is not used but is kept for possible later assignments
Input::Input() : xPos(0), yPos(0), zPos(0), roll(0), pitch(0), yaw(0), m_mlDown(false), part(InputNS::PART_1) {}


void Input::reset()
{
	m_mlDown = false;
	roll = pitch = yaw = 0;
	xPos = yPos = zPos = 0;
	part = InputNS::PART_1;
}

//keyboard will be used when glut registers a key input
void Input::keyboard(const unsigned char key, const int x, const int y)
{
	/****************************
	How To mess around with this program

	
	Camera movement
		movement up/ down = w, s
		movement left/ right = a d
		movement in / out = q, e
		roll =  u / o
		yaw =  j / l
		pitch i / k

	light movement
		move in x  f / h
		move in y g /t
		move in z r / y
	
	Color
	  v b n represent ambient, diffuse, specular

	Misc
		m make 150 more cubes! Go crazy and crash the program!
	******************************/

	switch(key)
	{

	/******************************
	Light Movement Keys
	*******************************/
	case 'f':
		lighting->updatePosition(vec4(0.1, 0, 0, 0));
		break;
	case 'h':
		lighting->updatePosition(vec4(-0.1, 0, 0, 0));
		break;
	case 'g':
		lighting->updatePosition(vec4(0, 0.1, 0, 0));
		break;
	case 't':
		lighting->updatePosition(vec4(0, -0.1, 0, 0));
		break;
	case 'r':
		lighting->updatePosition(vec4(0, 0, -0.1, 0));
		break;
	case 'y':
		lighting->updatePosition(vec4(0, 0, 0.1, 0));
		break;
	/**********************************
	Change the light source
	**********************************/
	case 'v':
		lighting->switchActiveLight(LightNS::AMBIENT);
		break;
	case 'b':
		lighting->switchActiveLight(LightNS::DIFFUSE);
		break;
	case 'n':
		lighting->switchActiveLight(LightNS::SPECULAR);
		break;

	/***********************************
	Camera Movement
	************************************/
	case 'e':
		yPos+= .01;
		translate *= Translate(0,.1,0);
		break;
	case 'q':
		yPos-= .01;
		translate *= Translate(0,-.1,0);
		break;
	case 'a':
		xPos-= -.01;
		translate *= Translate(-.1,0,0);
		break;
	case 'd':
		xPos+= .01;
		translate *= Translate(.1,0,0);
		break;
	case 'w':
		zPos += .01;
		translate *= Translate(0 , 0, .1);
		break;
	case 's':
		zPos-= .01;
		translate *= Translate(0, 0, -.1);
		break;
	case 'u':
		--roll;
		rotate *= RotateZ(-1);
		break;
	case 'o':
		++roll;
		rotate *= RotateZ(1);
		break;
	case 'j':
		--yaw;
		rotate *= RotateY(-1);
		break;
	case 'l':
		++yaw;
		rotate *= RotateY(1);
		break;
	//pitch
	case 'i':
		++pitch;
		rotate *= RotateX(1);
		break;
	case 'k':
		--pitch;
		rotate *= RotateX(-1);
		break;
	case 'm':
		makeFarTooManyCubes(150);
		break;
	case ' ':
		rotate = identity();
		translate = identity();
		reset();
		lighting->reset();
	}

	glutPostRedisplay();
}

//special keys will handle the arrow keys as freeglut handles
//them seperately from the other keyboard keys
void	Input::specialkeys(const int key, const int x, const int y)
{
	//up and down on arrow keys increase and decrease current active 
	//left and right cycle between x y and z on current light
	switch(key){
	case GLUT_KEY_UP:
		lighting->updateActiveLight(.1);
		break;	
	case GLUT_KEY_DOWN:
		lighting->updateActiveLight(-.1);
		break;
	case GLUT_KEY_LEFT:
		lighting->switchActiveRGB(-1);
		break;
	case GLUT_KEY_RIGHT:
		lighting->switchActiveRGB(1);
		break;
	}
	glutPostRedisplay();
}


//mouse will be used when we register a mouse input. 
void Input::mouse(const int button, const int state, const int x, const int y)
{
	/*******************************************
	Mouse Controlls
	left  == active x for lighting
	middle == active y for lighting
	right  == active z for lighting
	******************************************************************************************/
	if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
	{
		lighting->switchCoords(LightNS::X);
	}

	if(button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN)
	{
		lighting->switchCoords(LightNS::Y);

	}

	if(button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
	{
		lighting->switchCoords(LightNS::Z);
	}
}

