/*******************************************
Lighting.cpp contains the implementation
of class Light
********************************************/
#include "Lighting.h"


//constructor for light.  It will simply set up the variables for the class.
Light::Light()
{
	//force a recompute immedietely
	invalidated = true;
	activeLight = LightNS::AMBIENT;
	activeCoord = LightNS::X;
	rgb = 0;
}

//init will set up all the vectors that are needed and send over the initai lighting properties to the gpu
bool Light::init(GLuint program)
{
	position = vec4(2.0, 0.0, 20.0, 0.0 );
	ambient = vec4( .5, .5, .5, 1.0); 
	diffuse = vec4(.2, .5, .8, 1.0);
	specular = vec4(.5, .5, .5, 1.0);
	actX = ambient.x;
	actY = ambient.y;
	actZ = ambient.z;
	//store the program if we need it again.
	this->program = program;
	updateLighting();
	return true;
}

//reset the lighting.  Done when the user hits space on the keyboard.
void Light::reset()
{
	position = vec4(2.0, 0.0, 20.0, 0.0 );
	ambient = vec4( .5, .5, .5, 1.0); 
	diffuse = vec4(.2, .5, .8, 1.0);
	specular = vec4(.5, .5, .5, 1.0);
	invalidated = true;
	activeLight = LightNS::AMBIENT;
	activeCoord = LightNS::X;
	actX = ambient.x;
	actY = ambient.y;
	actZ = ambient.z;
	rgb = 0;
}

//updateLighting sends over the lighting information to the gpu
void Light::updateLighting()
{
	//note that since this is called every time we draw, we don't want to bother the gpu if we don't have to
	//so only update when we have to
	if(invalidated)
	{
		glUniform4fv( glGetUniformLocation(program, "ambientRaw"),
		  1, ambient );
		glUniform4fv( glGetUniformLocation(program, "diffuseRaw"),
		  1, diffuse );
		glUniform4fv( glGetUniformLocation(program, "specularRaw"),
		  1, specular );
	
		glUniform4fv( glGetUniformLocation(program, "lightPosition"),
		  1, position );


		invalidated = false;
	}
}

//the user wants to move to light so oblige him.  Note there's no error checking, as any valid vec4 will be ok, but it will absolutely be useless eventually
void Light::updatePosition(const vec4 newPosition)
{
	invalidated = true;
	position += newPosition;
}

//activelight will be updated by the shift amount.
void Light::updateActiveLight(GLfloat shift)
{
	//see if we're updating correctly.  Also not we clamp the input to 0.0 and 1.0 to make sure we are never out of lighting range.
	switch(rgb)
	{
	case LightNS::R:
		actX+= shift;
		if(actX > 1.0)
			actX = 1.0;
		else if(actX < 0)
			actX = 0.0;
		break;
	case LightNS::G:
		actY += shift;
		if(actY > 1.0)
			actY = 1.0;
		else if(actY < 0)
			actY = 0.0;
		break;
	case LightNS::B:
		actZ += shift;
		if(actZ > 1.0)
			actZ = 1.0;
		else if(actZ < 0)
			actZ = 0.0;
		break;
	default:
		std::cerr<<"Really bad stuff happened in updateActiveLight" << std::endl;
		return;
		break;
	}
	//go to the hidden second part of updateActiveLight();
	updateActiveLight();
}

//this part will handle the actual updating of the new vector for one of the
//three types of lighting.
void Light::updateActiveLight()
{
	switch(activeLight)
	{
	case LightNS::AMBIENT:
		ambient = vec4(actX, actY, actZ, 1.0);
		break;
	case LightNS::DIFFUSE:
		diffuse = vec4(actX, actY, actZ, 1.0);
		break;
	case LightNS::SPECULAR:
		specular = vec4(actX, actY, actZ, 1.0);
		break;
	default:
		std::cerr << "Bad new light";
		return;
		break;
	}
	invalidated = true;
}

//when the user chooses to switch their active light we will simply set it to the new active light
void Light::switchActiveLight(const int newLight)
{
	if(newLight == LightNS::AMBIENT || newLight == LightNS::DIFFUSE || newLight == LightNS::SPECULAR)
	{
		activeLight = newLight;
		
	}
	//change the active parameters 
	switch(activeLight)
	{
	case LightNS::AMBIENT:
		actX = ambient.x;
		actY = ambient.y;
		actZ = ambient.z;
		break;
	case LightNS::DIFFUSE:
		actX = diffuse.x;
		actY = diffuse.y;
		actZ = diffuse.z;
		break;
	case LightNS::SPECULAR:
		actX = specular.x;
		actY = specular.y;
		actZ = specular.z;
		break;
	default:
		std::cerr << "Bad new light";
		return;
		break;
	}

}

//switch coords will update the coord to the new one, based off what the user said through the keyboard
void Light::switchCoords(const int newCoord)
{
	if(newCoord == LightNS::X || newCoord == LightNS::Y || newCoord == LightNS::Z)
	{
		activeCoord = newCoord;
	}
}

//handle what color to modify when the user pressed up or down on the keyboard.
void Light::switchActiveRGB(const int shift)
{
	rgb+= shift;
	if(rgb < 0)
	{
		//shift 3 to get you back to 2 instead of 1.
		rgb += 3;
	}
	rgb = (rgb % 3);
}