/********************************************************
Lighting.h contains the definition of class Light

*********************************************************/
#ifndef LIGHTING_H
#define LIGHTING_H
#include "Angel.h"

/**************
Namespace LightNS will hold all the constants for this class.
This way we can reuse some of the one letter consts if we have to.
**************/
namespace LightNS{
	const int AMBIENT = 1;
	const int DIFFUSE = 2;
	const int SPECULAR = 3;
	const int X = 1;
	const int Y = 2;
	const int Z = 3;
	const int R = 0;
	const int G = 1;
	const int B = 2;
};


class Light
{
public:
		 Light();
	bool init(GLuint program);
	void updateLighting();
	void reset();
	void updatePosition(const vec4 newPosition);
	void updateCoords(const int newCoord);
	void updateActiveLight(const GLfloat shift);
	void switchActiveLight(const int newLight);
	void switchCoords(const int newCoord);
	void switchActiveRGB(const int shift);


private:
	void	updateActiveLight();
	GLuint	program;
	bool	invalidated; //if true we must recompute lighting in shaders
	vec4	position, diffuse, ambient, specular;
	GLfloat actX, actY, actZ; //holds the current updated active x y and z coords for the current type of lighting we are modifying.
	int		activeLight; //the current type of lighting that we're modifying
	int		activeCoord; //the current coordinate we are modifying for position
	int		rgb; // the current red green blue value we are modifying for the activelight.


};


#endif