/***********************************************************************
Daniel Greco
Contains the implementation of class points. 
***********************************************************************/
#include "Points.h"


//default constructor 
Points::Points() : purge(false), curPoints(0){}

//init will create all the necessary member variables.
bool Points::init()
{
	try
	{
		userPoints = new std::list<vec3>();
		colors = new std::list<vec3>();
		vectors = new VectorCoords();
		normals = new std::list<vec3>();
		uvs = new std::list<vec2>();
		shines = new std::list<float>();
	}
	catch(...)
	{
		return false;
	}
	return true;
}

//destructor for class points
Points::~Points()
{
	delete userPoints;
	delete colors;
	delete vectors;
	delete normals;
	delete uvs;
	delete shines;
}

//clear all the arrays that are in the class
void Points::clearArrays()
{
	userPoints->clear();
	colors->clear();
	vectors->clear();
	uvs->clear();
	normals->clear();
	shines->clear();
	//set up the vectors class again, resetiting the size of the array.

	curPoints = 0;

}

//addToList will take in a new vector point, as well as a color and will assign them
//to their respective arrays
bool Points::addToList(vec3 newItem, vec3 color)
{
	//make sure they are not null
	if(newItem == NULL || color == NULL)
	{
		return false;
	}
	//push back because we want the newest items to be in the back of the array, for drawing purposes
	userPoints->push_back(newItem);
	colors->push_back(color);
	return true;
}

//a more modern addToList, it now allows you to handle normals for lighting purposed
bool	Points::addToList(vec3 newItem, vec3 color, vec3 normals)
{
	//make sure no garbage is given
	if(normals == NULL)
		return false;

	this->normals->push_back(normals);
	
	return addToList(newItem, color);
}

//Whoa another moderner version this time supporting inserting uvs. Note this isn't used for this assignment,
//as we do not support textures.
bool	Points::addToList(vec3 newItem, vec3 color, vec3 normals, vec2 uvs)
{
	if(uvs == NULL)
	{
		return false;
	}
	this->uvs->push_back(uvs);
	return addToList(newItem, color, normals);
}

//addShine is handled seperately from the others as we are given one shine, and each one has to be duplicated 36 times 
//this was a last minute add, and could definitely be improved upon.
void	Points::addShine(const GLfloat shine)
{
	for(int i = 0; i < 36; ++i)
	{
		shines->push_back(shine);
	}
}

//purgequeue will be used to set up the arrays for drawing.
//it will also split the shapes
void Points::purgeQueue()
{
	//get the size of the vertices
	int size = userPoints->size();
	
	//if the size is zero, very bad things will happen so don't even try to go forward.
	if(size == 0)
	{
		return;
	}
	//init the vector array with the current size
	vectors->init(size);

	vec3 curVec;
	int i = 0;

	//run through the vector array until we're at the end, or we hit the size
	for(std::list<vec3>::iterator iter = userPoints->begin(); iter != userPoints->end(); ++iter)
	{
		curVec = *iter;
		vectors->getVecArray()[i++] = curVec;
	}

	i = 0;
	//run through the color array until we're at the end or we hit the size.
	for(std::list<vec3>::iterator iter = colors->begin(); iter != colors->end(); ++iter)
	{
		curVec = *iter;
		
		vectors->getColorArray()[i++] = curVec;
	}

	i=0;
	//run through the normals array until we're at the end or we hit the size.
	for(std::list<vec3>::iterator iter = normals->begin(); iter != normals->end(); ++iter)
	{
		curVec = *iter;
		
		vectors->getNormalArray()[i++] = curVec;
	}

	vec2 curVec2;
	i = 0;
	//run through the uvs array until we're at the end or we hit the size. Note we are now using vec2's so we have to change
	//our temp variable
	for(std::list<vec2>::iterator iter = uvs->begin(); iter != uvs->end(); ++iter)
	{
		curVec2 = *iter;
		vectors->getUVSArray()[i++] = curVec2;
	}

	float curShine;
	i = 0;
	//run through the shines array until we're at the end or we hit the size. Note we are now using floats's so we have to change
	//our temp variable
	for(std::list<float>::iterator iter = shines->begin(); iter != shines->end(); ++iter)
	{
		curShine = *iter;
		vectors->getShineArray()[i++] = curShine;
	}
}
